unit W32filop;

(*
  Provides 16-bit Delphi 1.0 access to the Windows 95/NT 32-bit
  API.

  Written 03/18/96 by John J. Newlin, CIS 71535,665.  This source
  code, like the supporting units LONGNAME.PAS and CALL32NT.PAS is
  released to the public domain.  The author of this unit makes no
  guarantees, expressed or implied, and cautions you to use at your
  own risk.

  This unit puts wrappers on the routines in LONGNAME.PAS and
  CALL32NT.PAS {written by Christian Ghisler, CIS: 100332,1175}.

  These routines have been tested successfully on floppy and hard
  drives on a Micron P-100 running Windows95.  It is suggested that
  you test them carefully before incorporating them into your code.

  Note that you can add just about any WIN32 API function to this
  library by follwing the rules set forth in the file USAGE.TXT.

  ENJOY!
*)

interface

uses
  WinTypes, WinProcs, SysUtils, Dialogs, LongName,Call32NT;

type
  ActionType32 = (read32,write32);

function CopyFile32(Const Source,Dest : string; Overwrite : boolean) : LongBool;
function MoveFile32(Const OldName,NewName : string) : LongBool;
function BackupFile32(Var FullPath,NewName : string) : LongBool;
function DeleteFile32(Const FileName : string) : LongBool;
function FindFirst32(Const Path : string; Var Srec :WIN32_FIND_DATA) : longint;
function FindNext32(FindHandle : longint; Var Srec :WIN32_FIND_DATA) : longBool;
function FindClose32(FindHandle:longint) : LongBool;
function MkDir32(Const DirName : string) : LongBool;
function RmDir32(Const DirName : string) : Longbool;
function GetFileDateTime32(Var Rec:WIN32_FIND_DATA; Which:byte):TDateTime;
function DosShortName32(const FileName : string) : string;
function ResetFile32(Const FileName : string) : longint;
function ReWriteFile32(Const FileName : string) : longint;
function OpenFile32(Const FileName : string) : longint;
function CloseFile32(Handle:longint) : LongBool;
function BlockRead32(Handle:longint; Var Buffer; Count:word; Var BytesRead : longint) : LongBool;
function BlockWrite32(Handle:longint; Var Buffer; Count:word; Var BytesRead : longint) : LongBool;
function SeekFile32(Handle,Position : longint) : Longint;
function FilePos32(Handle : longint) : longint;

implementation

type
  PCharBuffer = array[0..254] of char;

function ResetFile32(Const FileName : string) : longint;
var
  Fname : PCharBuffer;
begin
  StrPCopy(Fname,FileName);
  Result := W32CreateFile(Fname,GENERIC_READ,FILE_SHARE_READ,nil,
                        OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0,
                        id_W32CreateFile);
end;

function ReWriteFile32(Const FileName : string) : longint;
var
  Fname : PCharBuffer;
begin
  StrPCopy(Fname,FileName);
  Result := W32CreateFile(FName,GENERIC_WRITE,0,nil,
                        OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,0,
                        id_W32CreateFile);
end;

function OpenFile32(Const FileName : string) : longint;
var
  Fname : PCharBuffer;
begin
  StrPCopy(Fname,FileName);
  Result := W32CreateFile(Fname,GENERIC_READ or GENERIC_WRITE,
                          FILE_SHARE_READ or FILE_SHARE_WRITE,
                          Nil,
                          OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,
                          0,
                          id_W32CreateFile);
end;

function SeekFile32(Handle,Position : longint) : Longint;
begin
  Result := W32SetFilePointer(Handle,Position,nil,FILE_BEGIN,id_W32SetFilePointer);
end;

function FilePos32(Handle : longint) : longint;
begin
  Result := W32SetFilePointer(Handle,0,nil,FILE_CURRENT,id_W32SetFilePointer);
end;

function CloseFile32(Handle:longint) : LongBool;
begin
  Result := W32CloseHandle(Handle,id_W32CloseHandle);
end;

function BlockRead32(Handle:longint; Var Buffer; Count:word; Var BytesRead : longint) : LongBool;
begin
  Result := W32ReadFile(Handle,Buffer,Count,BytesRead,nil,id_W32ReadFile);
end;

function BlockWrite32(Handle:longint; Var Buffer; Count:word; Var BytesRead : longint) : LongBool;
begin
  Result := W32WriteFile(Handle,Buffer,Count,BytesRead,nil,id_W32WriteFile);
end;

function CopyFile32(Const Source,Dest : string; Overwrite : boolean) : LongBool;
const
  BuffSize = 48 * 1024; {must not exceed 64k!}

var
  CopyBuffer : pointer;
  src,des : PCharBuffer;
  srcH,desH : longint;
  ResR,ResW : longint;
  ToRead,BytesRead,Written : longint;
  ok : longbool;
  CreateCriteria : longint;

begin
  Result := true;
  Try
    GetMem(CopyBuffer,BuffSize);
    except result := false;
  End;
  if not result then exit;
  if Overwrite then CreateCriteria := CREATE_ALWAYS else
    CreateCriteria := CREATE_NEW;
  Result := false;
  StrPCopy(Src,Source);
  StrPCopy(Des,Dest);
  SrcH := W32CreateFile(Src,GENERIC_READ,FILE_SHARE_READ,nil,
                        OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0,
                        id_W32CreateFile);
  if SrcH = -1 then
    begin
      Result := false;
      FreeMem(CopyBuffer,BuffSize);
      exit;
    end;
  DesH := W32CreateFile(Des,GENERIC_WRITE,0,nil,
                        CreateCriteria,FILE_ATTRIBUTE_NORMAL,0,
                        id_W32CreateFile);
  if DesH = -1 then
    begin
      W32CloseHandle(SrcH,id_W32CloseHandle);
      result := false;
      FreeMem(CopyBuffer,BuffSize);
      exit;
    end;
  Repeat
    Ok := W32ReadFile(SrcH,CopyBuffer^,BuffSize,BytesRead,nil,id_W32ReadFile);
    if Ok then Ok := W32WRiteFile(DesH,CopyBuffer^,BytesRead,Written,nil,id_W32WriteFile);
  Until (BytesRead = 0) or not Ok;
  Result := ok;
  FreeMem(CopyBuffer,BuffSize);
  W32CloseHandle(DesH,id_W32CloseHandle);
  W32CloseHandle(SrcH,id_W32CloseHandle);
end;

function MoveFile32(Const OldName,NewName : string) : LongBool;
var
  OldFile,NewFile : PCharBuffer;
begin
  StrPCopy(OldFile,OldName);
  StrPCopy(NewFile,NewName);
  Result := W32MoveFileEx(OldFile,NewFile,MOVEFILE_REPLACE_EXISTING,id_W32MoveFileEx);
end;

function BackupFile32(var FullPath,NewName : string) : LongBool;
var
  NewPath : string;
  Orig,NewFile : PCharBuffer;
begin
  result := false;
  NewPath := ExtractFilePath(FullPath)+ ExtractFileName(NewName);
  if LowerCase(FullPath) = LowerCase(NewPath) then exit;
  StrPCopy(Orig,FullPath);
  StrPCopy(NewFile,NewPath);
  Result := CopyFile32(FullPath,NewPath,true);
end;

function DeleteFile32(Const FileName : string) : LongBool;
var
  FName : PCharBuffer;
begin
  StrPCopy(Fname,FileName);
  Result := W32DeleteFile(Fname,id_W32DeleteFile);
end;

function FindFirst32(Const Path : string; Var Srec :WIN32_FIND_DATA) : longint;
var
  Lpath : PCharBuffer;
begin
  StrPCopy(Lpath,Path);
  Result := W32FindFirstFile(Lpath,Srec,id_W32FindFirstFile);
end;

function FindNext32(FindHandle : longint; Var Srec :WIN32_FIND_DATA) : longBool;
begin
  Result := W32FindNextFile(FindHandle,Srec,id_W32FindNextFile);
end;

function FindClose32(FindHandle:longint) : LongBool;
begin
  Result := W32FindClose(FindHandle,id_W32FindClose);
end;

function MkDir32(Const DirName : string) : LongBool;
var
  NewDir : PCharBuffer;
begin
  StrPCopy(NewDir,DirName);
  Result := W32CreateDirectory(NewDir,nil,id_W32CreateDirectory);
end;

function RmDir32(Const DirName : string) : Longbool;
Var
  OldDir : PCharBuffer;
begin
  StrPCopy(OldDir,DirName);
  Result := W32RemoveDirectory(OldDir,id_W32RemoveDirectory);
end;

function SetCurrDir32(Var Dir : string) : LongBool;
var
  CurrDir : PCharBuffer;
begin
  StrPCopy(CurrDir,Dir);
  Result := W32SetCurrentDirectory(CurrDir,id_W32SetCurrentDirectory);
end;

function GetCurrDir32 : string;
var
  CurrDir : PCharBuffer;
begin
  W32GetCurrentDirectory(sizeOf(CurrDir),CurrDir,id_W32GetCurrentDirectory);
  Result := StrPas(CurrDir);
end;

function GetFileDateTime32(Var Rec:WIN32_FIND_DATA; Which:byte):TDateTime;
var
  Dtime,Ddate : word;
  Dt,x : longint;
  TimeRec : tFileTime;
begin
  x := rec.ftlastwritetime[0];
  Result := StrtoDate('01/01/80');
  with Rec do case Which of
  1 :
    begin
      TimeRec.dwLowDateTime := ftCreationTime[0];
      TimeRec.dwHighDateTime := ftCreationTime[1];
    end;
  2 :
    begin
      TimeRec.dwLowDateTime :=  ftLastAccessTime[0];
      TimeRec.dwHighDateTime := ftLastAccessTime[1];
    end;
  3 :
    begin
      TimeRec.dwLowDateTime := ftLastWriteTime[0];
      TimeRec.dwHighDateTime := ftLastWriteTime[1];
    end;
    Else exit;
  End; {of case}
  if W32FileTimeToDosTime(TimeRec,Ddate,Dtime,id_W32FileTimeToDosTime) then
    begin
      Dt := Ddate;
      dt := (dt shl 16) + Dtime;
      Try
        Result := FileDateToDateTime(dt);
        except
      End;
      W32DosTimeToFileTime(Ddate,Dtime,TimeRec,id_W32DosTimeToFileTime);
    end;
end;

function DosShortName32(const FileName : string) : string;
var
  Srec : WIN32_FIND_DATA;
  FHandle,Result32 : longint;
  ShortName : string[13];
begin
  Result := '';
  Result32 := FindFirst32(FileName,Srec);
  if Result32 = -1 then exit;
  FindClose32(Result32);
  if SRec.cAlternateFileName[0] <> #0 then
    ShortName := StrPas(Srec.cAlternateFileName) else
      ShortName := StrPas(Srec.cFileName);
  Result := ShortName;
end;

END.

(*
dwDesiredAccess

Specifies the type of access to the file or other object. An application
can obtain read access, write access, read-write access, or device
query access. You can use the following flag constants to build a value
for this parameter. Both GENERIC_READ and GENERIC_WRITE must be set to
obtain read-write access:

Value	Meaning
0	        Allows an application to query device attributes without
                actually accessing the device.
GENERIC_READ	Specifies read access to the file. Data can be read from
                the file and the file pointer can be moved.
GENERIC_WRITE	Specifies write access to the file. Data can be written to
                the file and the file pointer can be moved.
dwShareMode

Specifies how this file can be shared. This parameter must be some combination of the following values:

Value	         Meaning
0		 Prevents the file from being shared.
FILE_SHARE_READ	 Other open operations can be performed on the file for
		 read access. If the CreateFile function is opening the client
                 end of a mailslot, this flag is specified.
FILE_SHARE_WRITE Other open operations can be performed on the file for write access.

lpSecurityAttributes

Points to a SECURITY_ATTRIBUTES structure that specifies the security
attributes for the file. The file system must support this parameter
for it to have an effect.

dwCreationDistribution

Specifies which action to take on files that exist, and which action to
take when files do not exist. This parameter must be one of the following
values:

Value			Meaning
CREATE_NEW		Creates a new file. The function fails if the
			specified file already exists.
CREATE_ALWAYS		Creates a new file. The function overwrites the file
			if it exists.
OPEN_EXISTING		Opens the file. The function fails if the file does
			not exist.
OPEN_ALWAYS		Opens the file, if it exists. If the file does not
			exist, the function creates the file as if
                        dwCreationDistribution were CREATE_NEW.
TRUNCATE_EXISTING	Opens the file. Once opened, the file is truncated
			so that its size is zero bytes. The calling process
                        must open the file with at least GENERIC_WRITE access. The function fails if the file does not exist.

dwFlagsAndAttributes

Specifies the file attributes and flags for the file.
Any combination of the following attributes is acceptable, except all other file attributes override FILE_ATTRIBUTE_NORMAL.

Attribute		Meaning
FILE_ATTRIBUTE_ARCHIVE	The file is an archive file. Applications use this
			attribute to mark files for backup or removal.
FILE_ATTRIBUTE_COMPRESSED The file or directory is compressed. For a
			file, this means that all of the data in the file is
                        compressed. For a directory, this means that
                        compression is the default for newly created files
                        and subdirectories.
FILE_ATTRIBUTE_NORMAL	The file has no other attributes set. This attribute
			is valid only if used alone.
FILE_ATTRIBUTE_HIDDEN	The file is hidden. It is not to be included in an
			ordinary directory listing.
FILE_ATTRIBUTE_READONLY	The file is read only. Applications can read the file
			but cannot write to it or delete it.
FILE_ATTRIBUTE_SYSTEM	The file is part of or is used exclusively by the
			operating system.

Any combination of the following flags is acceptable.

Flag			Meaning
FILE_FLAG_WRITE_THROUGH Instructs the operating system to write through any
			intermediate cache and go directly to the file. The
                        operating system can still cache write operations,
                        but cannot lazily flush them.
FILE_FLAG_OVERLAPPED	Instructs the operating system to initialize the file,
			so ReadFile, WriteFile, ConnectNamedPipe, and
                        TransactNamedPipe operations that take a significant
                        amount of time to process return ERROR_IO_PENDING.
                        When the operation is finished, an event is set to
                        the signaled state.  When you specify
                        FILE_FLAG_OVERLAPPED, the ReadFile and WriteFile
                        functions must specify an OVERLAPPED structure. That
                        is, when FILE_FLAG_OVERLAPPED is specified, an
                        application must perform overlapped reading and
                        writing. When FILE_FLAG_OVERLAPPED is specified, the
                        operating system does not maintain the file pointer.
                        The file position must be passed as part of the
                        lpOverlapped parameter (pointing to an OVERLAPPED
                        structure) to the ReadFile and WriteFile functions.
			This flag also enables more than one operation to be
                        performed simultaneously with the handle (a
                        simultaneous read and write operation, for example).
FILE_FLAG_NO_BUFFERING  Instructs the operating system to open the file with
			no intermediate buffering or caching. This can provide
                        performance gains in some situations. An application
                        must meet certain requirements when working with files
                        opened with FILE_FLAG_NO_BUFFERING. File access must
                        begin at offsets within the file that are integer
                        multiples of the volume's sector size. File access
                        must be for numbers of bytes that are integer
                        multiples of the volume's sector size. For example,
                        if the sector size is 512 bytes, an application can
                        request reads and writes of 512, 1024, or 2048 bytes,
                        but not of 335, 981, or 7171 bytes. 	Buffer
                        addresses for read and write operations must be
                        aligned on addresses in memory that are integer
                        multiples of the volume's sector size. An application
                        can determine a volume's sector size by calling the
                        GetDiskFreeSpace function.
FILE_FLAG_RANDOM_ACCESS	Indicates that the file is accessed randomly. Windows
			uses this flag to optimize file caching.
FILE_FLAG_SEQUENTIAL_SCAN  Indicates that the file is to be accessed
			sequentially from beginning to end. Windows uses this
                        flag to optimize file caching. If an application
                        moves the file pointer for random access, optimum
                        caching may not occur; however, correct operation is
                        still guaranteed. Specifying this flag can increase
                        performance for applications that read large files
                        using sequential access. Performance gains can be even
                        more noticeable for applications that read large files
                        mostly sequentially, but occasionally skip over small
                        ranges of bytes.
FILE_FLAG_DELETE_ON_CLOSE  Indicates that the operating system is to delete
			the file immediately after all of its handles have
                        been closed.If you use this flag when you call
                        CreateFile, then open the file again, and then close
                        the handle for which you specified
                        FILE_FLAG_DELETE_ON_CLOSE, the file will not be
                        deleted until after you have closed the second and
                        any other handle to the file.
FILE_FLAG_BACKUP_SEMANTICS  Windows NT only: Indicates that the file is being
			opened or created for a backup or restore operation.
                        The operating system ensures that the calling process
                        overrides file security checks, provided it has the
                        necessary permission to do so. The relevant
                        permissions are SE_BACKUP_NAME and SE_RESTORE_NAME. A
                        Windows NT application can also set this flag to
                        obtain a handle to a directory. A directory handle
                        can be passed to some Win32 functions in place of a
                        file handle.
FILE_FLAG_POSIX_SEMANTICS  Indicates that the file is to be accessed according
			 to POSIX rules. This includes allowing multiple files
                         with names, differing only in case, for file systems
                         that support such naming. Use care when using this
                         option because files created with this flag may not
                         be accessible by applications written for MS-DOS,
                         Windows 3.x, or Windows NT.
*)
